package net.tomatentum.cutin;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import net.tomatentum.cutin.util.ReflectionUtil;


public class ReflectedMethodFactoryImpl<I extends Object> implements ReflectedMethodFactory<I> {

    private Logger logger = LoggerFactory.getLogger(getClass());

    private List<Factory<I>> factories;

    public ReflectedMethodFactoryImpl() {
        this(new ArrayList<>());
    }

    public ReflectedMethodFactoryImpl(List<Factory<I>> factories) {
        this.factories = factories;
    }

    @Override
    public Optional<ReflectedMethod<I>> produce(Method method, Object containingClass) {
        Optional<ReflectedMethod<I>> rmethod = this.factories.stream()
            .map(f -> factoryProduce(f, method, containingClass))
            .filter(Optional::isPresent)
            .map(Optional::get)
            .findFirst();

        if (rmethod.isEmpty()) {
            logger.debug("Could not produce a ReflectedMethod for Method {}", ReflectionUtil.getFullMethodName(method));
        }

        return rmethod;
    }

    @Override
    public ReflectedMethodFactory<I> addFactory(Factory<I> factory) {
        this.factories.add(factory);
        return this;
    }

    private Optional<ReflectedMethod<I>> factoryProduce(Factory<I> factory, Method method, Object containingClass) {
        List<MethodParser> parser = new ArrayList<>();
        Optional<ReflectedMethod<I>> m = factory.produce(method, containingClass);
        m.ifPresent(x -> {
            factory.addParser(x, parser); 
            parser.forEach(MethodParser::parse);
        });
        return m;
    }
    
}
